/*!
 * @file signal_connector.h
 * @brief Definition of a SignalConnector class
*/

#ifndef _SIGNAL_CONNECTOR_H
#define _SIGNAL_CONNECTOR_H

#include "executor/executor.h"

namespace OrxGui
{

  //////////////// TO BE IGNORED BY YOU /////
#define DeclareSignalBegin(SignalName) \
  public: \
   void signal_ ## connect ## SignalName(const SignalConnector& connector) { \
     SignalName ## connected.push_back(connector); \
}\
   Signal& getSignalVector_##SignalName () { return this->SignalName ## connected; }; \
  private:

#define DeclareSignalEnd(SignalName) \
  Signal SignalName ## connected
/////////////////////////////////////////////


  /**
   * @brief declares a new Signal.
   * @param SignalName the Name of the Signal.
   */
#define DeclareSignal0(SignalName) \
  DeclareSignalBegin(SignalName) \
   void  SignalName () { \
     for (unsigned int i = 0; i < SignalName ## connected .size(); i++) \
       SignalName ## connected[i] (); \
   }\
     DeclareSignalEnd(SignalName)


  /**
 * @brief declares a new Signal.
 * @param SignalName the Name of the Signal.
   */
#define DeclareSignal1(SignalName, param0) \
  DeclareSignalBegin(SignalName) \
   void  SignalName (param0 val0) { \
     for (unsigned int i = 0; i < SignalName ## connected .size(); i++) \
       SignalName ## connected[i] (val0); \
    }\
    DeclareSignalEnd(SignalName)


   /**
    * @brief selects a Signal.
    * @param SignalName the Signal to be retrieved.
    */
#define SIGNAL(Object, SignalName) \
    Object->getSignalVector_##SignalName()

/**
 * @brief defines a Slot, the sink of a Signal.
 * @param Class the Class the Slot belongs to.
 * @param function the Function to Connect to.
 */
#define SLOT(Class, function) \
    createExecutor<Class>(&Class::function)

/**
 * @brief emits function
 */
#define emit(function) function

  //! A class for Conncting Signals to Objects, inside of the GUI
  class SignalConnector
  {
  public:
    SignalConnector();
    SignalConnector(BaseObject* object, const Executor* exec);
    SignalConnector(const SignalConnector& signalConnector);
    ~SignalConnector();

    SignalConnector& operator=(const SignalConnector& signalConnector);

    void operator()() const;
    void operator()(const MultiType& value0) const;

    bool isValid() const { return (this->object && this->exec); };
    bool isClean() const { return (this->object == NULL || this->exec == NULL); }


  private:
    BaseObject*          object;         //!< The object to call.
    const Executor*      exec;           //!< The Executor, that will be called, on object.
  };

  typedef std::vector<SignalConnector> Signal;
  typedef Executor*                    Slot;

}
#endif /* _SIGNAL_CONNECTOR_H */
